
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#ifndef	DATA_NUMBER_LIMIT
#define DATA_NUMBER_LIMIT	1024
#endif

#ifndef	BOUNDARY_CONDITION
#define	BOUNDARY_CONDITION	2.0
#endif

static void	Error(msg)
char		*msg;
{
	fputs(msg,stderr); fputc('\n',stderr); exit(1);
}

static void	Print(N,x,y,h,v,a,b,g,p,q,d)
int		N;
double		*x,*y,*h,*v,*a,*b,*g,*p,*q,*d;
{
	int	N1,n;
	double	c;

	switch(N) {
	    case 0 :
		return;
	    case 1 :
		printf("%lf\t%lf\t0\t0\t0\n",x[0],y[0]); return;
	    case 2 :
		printf("%lf\t%lf\t%lf\t0\t0\n",
		       x[0],y[0],(y[1]-y[0])/(x[1]-x[0])); return;
	}
	N1=N-1;

	for (n=1; n<=N1; n++) {
	    h[n]=x[n]-x[n-1];
	    v[n]=y[n]-y[n-1];
	}

	for (n=1; n<N1; n++) {
	    a[n]=h[n+1]/(h[n]+h[n+1]);
	    b[n]=1.0-a[n];
	    g[n]=3.0*(a[n]*v[n]/h[n]+b[n]*v[n+1]/h[n+1]);
	}

	b[ 0]=(BOUNDARY_CONDITION); g[ 0]=(BOUNDARY_CONDITION+2.0)*v[ 1]/h[ 1];
	a[N1]=(BOUNDARY_CONDITION); g[N1]=(BOUNDARY_CONDITION+2.0)*v[N1]/h[N1];

	p[0]=2.0; q[0]=g[0];
	for (n=1; n<=N1; n++) {
	    c=a[n]/p[n-1]; p[n]=2.0 -b[n-1]*c;
			   q[n]=g[n]-q[n-1]*c;
	}
	d[N1]=q[N1]/p[N1];
	for (n=N1-1; n>=0; n--) d[n]=(q[n]-b[n]*d[n+1])/p[n];

	for (n=0; n<N1; n++)
	    printf("%lf\t%lf\t%lf\t%lf\t%lf\n",
		   x[n],
		   y[n],
		   d[n],
		   (3.0*v[n+1]-(d[n+1]+2.0*d[n])*h[n+1])/(h[n+1]*h[n+1]),
		   ((d[n+1]+d[n])*h[n+1]-2.0*v[n+1])/(h[n+1]*h[n+1]*h[n+1]));
}

#define ALLOC(num)	(double *)malloc((num)*sizeof(double))

int	main(argc,argv)
int	argc;
char	**argv;
{
	int	N,DNL=DATA_NUMBER_LIMIT;
	double	*x,*y,*h,*v,*a,*b,*g,*p,*q,*d,e,xN,yN;

	if (argc!=1 && argc!=2) Error("usage : sderiv {dataNumberLimit}");

	if (argc==2) DNL=atoi(argv[1]);

	if ((x=ALLOC(DNL))==NULL ||
	    (y=ALLOC(DNL))==NULL ||
	    (h=ALLOC(DNL))==NULL ||
	    (v=ALLOC(DNL))==NULL ||
	    (a=ALLOC(DNL))==NULL ||
	    (b=ALLOC(DNL))==NULL ||
	    (g=ALLOC(DNL))==NULL ||
	    (p=ALLOC(DNL))==NULL ||
	    (q=ALLOC(DNL))==NULL ||
	    (d=ALLOC(DNL))==NULL) Error("memory allocation error.");

	for (e=(-1.0), N=0; scanf("%lf %lf",&xN,&yN)==2; N++, e=xN) {
	    if (yN<=0.0) Error("illegal data value.");

	    if (xN==e) {
		Print(N,x,y,h,v,a,b,g,p,q,d); N=0;
	    }
	    if (N==DNL) Error("too many number of data.");

	    x[N]=xN; y[N]=log(yN);
	}
	Print(N,x,y,h,v,a,b,g,p,q,d); return 0;
}
